// OverlayPlugin API setup
let inited = false;
let wsUrl = null;
let ws = null;
let queue = [];
let rseqCounter = 0;
const responsePromises = {};
const subscribers = {};
const sendMessage = (msg, cb) => {
    if (ws) {
        if (queue)
            queue.push(msg);
        else
            ws.send(JSON.stringify(msg));
    }
    else {
        if (queue)
            queue.push([msg, cb]);
        else
            window.OverlayPluginApi.callHandler(JSON.stringify(msg), cb);
    }
};
const processEvent = (msg) => {
    init();
    const subs = subscribers[msg.type];
    subs?.forEach((sub) => {
        try {
            sub(msg);
        }
        catch (e) {
            console.error(e);
        }
    });
};
export const dispatchOverlayEvent = processEvent;
export const addOverlayListener = (event, cb) => {
    init();
    if (!subscribers[event]) {
        subscribers[event] = [];
        if (!queue) {
            sendMessage({
                call: 'subscribe',
                events: [event],
            });
        }
    }
    subscribers[event]?.push(cb);
};
export const removeOverlayListener = (event, cb) => {
    init();
    if (subscribers[event]) {
        const list = subscribers[event];
        const pos = list?.indexOf(cb);
        if (pos !== undefined && pos > -1)
            list?.splice(pos, 1);
    }
};
const callOverlayHandlerInternal = (_msg) => {
    init();
    const msg = {
        ..._msg,
        rseq: 0,
    };
    let p;
    if (ws) {
        msg.rseq = rseqCounter++;
        p = new Promise((resolve, reject) => {
            responsePromises[msg.rseq] = { resolve: resolve, reject: reject };
        });
        sendMessage(msg);
    }
    else {
        p = new Promise((resolve, reject) => {
            sendMessage(msg, (data) => {
                if (data === null) {
                    resolve(data);
                    return;
                }
                const parsed = JSON.parse(data);
                if (parsed['$error'])
                    reject(parsed);
                else
                    resolve(parsed);
            });
        });
    }
    return p;
};
const callOverlayHandlerOverrideMap = {};
export const callOverlayHandler = (_msg) => {
    init();
    // If this `as` is incorrect, then it will not find an override.
    // TODO: we could also replace this with a type guard.
    const type = _msg.call;
    const callFunc = callOverlayHandlerOverrideMap[type] ?? callOverlayHandlerInternal;
    // The `IOverlayHandler` type guarantees that parameters/return type match
    // one of the overlay handlers.  The OverrideMap also only stores functions
    // that match by the discriminating `call` field, and so any overrides
    // should be correct here.
    // eslint-disable-next-line max-len
    // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-argument
    return callFunc(_msg);
};
export const setOverlayHandlerOverride = (type, override) => {
    if (!override) {
        delete callOverlayHandlerOverrideMap[type];
        return;
    }
    callOverlayHandlerOverrideMap[type] = override;
};
export const init = () => {
    if (inited)
        return;
    if (typeof window !== 'undefined') {
        wsUrl = new URLSearchParams(window.location.search).get('OVERLAY_WS');
        if (wsUrl !== null) {
            const connectWs = function (wsUrl) {
                ws = new WebSocket(wsUrl);
                ws.addEventListener('error', (e) => {
                    console.error(e);
                });
                ws.addEventListener('open', () => {
                    console.log('Connected!');
                    const q = queue ?? [];
                    queue = null;
                    sendMessage({
                        call: 'subscribe',
                        events: Object.keys(subscribers),
                    });
                    for (const msg of q) {
                        if (!Array.isArray(msg))
                            sendMessage(msg);
                    }
                });
                ws.addEventListener('message', (_msg) => {
                    try {
                        if (typeof _msg.data !== 'string') {
                            console.error('Invalid message data received: ', _msg);
                            return;
                        }
                        const msg = JSON.parse(_msg.data);
                        const promiseFuncs = msg?.rseq !== undefined ? responsePromises[msg.rseq] : undefined;
                        if (msg.rseq !== undefined && promiseFuncs) {
                            if (msg['$error'])
                                promiseFuncs.reject(msg);
                            else
                                promiseFuncs.resolve(msg);
                            delete responsePromises[msg.rseq];
                        }
                        else {
                            processEvent(msg);
                        }
                    }
                    catch (e) {
                        console.error('Invalid message received: ', _msg);
                        return;
                    }
                });
                ws.addEventListener('close', () => {
                    queue = null;
                    console.log('Trying to reconnect...');
                    // Don't spam the server with retries.
                    window.setTimeout(() => {
                        connectWs(wsUrl);
                    }, 300);
                });
            };
            connectWs(wsUrl);
        }
        else {
            const waitForApi = function () {
                if (!window.OverlayPluginApi?.ready) {
                    window.setTimeout(waitForApi, 300);
                    return;
                }
                const q = queue ?? [];
                queue = null;
                window.__OverlayCallback = processEvent;
                sendMessage({
                    call: 'subscribe',
                    events: Object.keys(subscribers),
                });
                for (const item of q) {
                    if (Array.isArray(item))
                        sendMessage(item[0], item[1]);
                }
            };
            waitForApi();
        }
        // Here the OverlayPlugin API is registered to the window object,
        // but this is mainly for backwards compatibility. For cactbot's built-in files,
        // it is recommended to use the various functions exported in resources/overlay_plugin_api.ts.
        /* eslint-disable deprecation/deprecation */
        window.addOverlayListener = addOverlayListener;
        window.removeOverlayListener = removeOverlayListener;
        window.callOverlayHandler = callOverlayHandler;
        window.dispatchOverlayEvent = dispatchOverlayEvent;
        /* eslint-enable deprecation/deprecation */
    }
    inited = true;
};
//# sourceMappingURL=overlay_plugin_api.js.map