import NetRegexes, { commonNetRegex } from '../../../resources/netregexes';
import { UnreachableCode } from '../../../resources/not_reached';
import { LocaleNetRegex } from '../../../resources/translations';
export const querySelectorSafe = (node, sel) => {
    const ret = node.querySelector(sel);
    if (!(ret instanceof HTMLElement))
        throw new UnreachableCode();
    return ret;
};
export const querySelectorAllSafe = (node, sel) => {
    const ret = [...node.querySelectorAll(sel)].map((elem) => {
        if (!(elem instanceof HTMLElement))
            throw new UnreachableCode();
        return elem;
    });
    return ret;
};
export const getTemplateChild = (node, sel) => {
    const template = querySelectorSafe(node, sel);
    if (!(template instanceof HTMLTemplateElement))
        throw new UnreachableCode();
    const ret = template.content.firstElementChild;
    if (!ret)
        throw new UnreachableCode();
    if (!(ret instanceof HTMLElement))
        throw new UnreachableCode();
    return ret;
};
export const cloneSafe = (node) => {
    const cloned = node.cloneNode(true);
    if (!(cloned instanceof HTMLElement))
        throw new UnreachableCode();
    return cloned;
};
export default class EmulatorCommon {
    static cloneData(data, exclude = ['options', 'party']) {
        const ret = {};
        // Use extra logic for top-level extend for property exclusion
        // This cut the execution time of this code from 41,000ms to 50ms when parsing a 12 minute pull
        for (const i in data) {
            if (exclude.includes(i))
                continue;
            if (typeof data[i] === 'object') {
                // Cloning any. See DataType definition above for reasoning.
                // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                ret[i] = EmulatorCommon._cloneData(data[i]);
                continue;
            }
            // Assignment of any to any. See DataType definition above for reasoning.
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            ret[i] = data[i];
        }
        return ret;
    }
    static _cloneData(data) {
        if (typeof data === 'object') {
            if (Array.isArray(data)) {
                const ret = [];
                for (let i = 0; i < data.length; ++i) {
                    // Don't clone completely empty entries in the array. This avoids issues
                    // with code like the following:
                    // let z = [];
                    // z[parseInt(matches.id)] = matches;
                    // `z` now has 0x40000000 "empty" entries that would otherwise be cloned
                    if (!(i in data))
                        continue;
                    // Cloning any. See DataType definition above for reasoning.
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                    ret[i] = EmulatorCommon._cloneData(data[i]);
                }
                return ret;
            }
            if (data === null)
                return null;
            if (data instanceof RegExp)
                return new RegExp(data);
            const ret = {};
            for (const i in data) {
                // Cloning any. See DataType definition above for reasoning.
                // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                ret[i] = EmulatorCommon._cloneData(data[i]);
            }
            return ret;
        }
        return data;
    }
    static doesLineMatch(line, regexes) {
        if (regexes instanceof RegExp)
            return regexes.exec(line);
        for (const langStr in regexes) {
            const lang = langStr;
            const res = regexes[lang].exec(line);
            if (res) {
                if (res.groups)
                    res.groups.language = lang;
                return res;
            }
        }
        return null;
    }
    static matchStart(line) {
        let res;
        // Currently all of these regexes have groups if they match at all,
        // but be robust to that changing in the future.
        res = EmulatorCommon.doesLineMatch(line, EmulatorCommon.countdownRegexes);
        if (res) {
            return {
                StartIn: (parseInt(res.groups?.time ?? '0') * 1000).toString(),
                StartType: 'Countdown',
                language: res.groups?.language ?? undefined,
            };
        }
        res = EmulatorCommon.doesLineMatch(line, EmulatorCommon.sealRegexes);
        if (res) {
            return {
                StartIn: '0',
                StartType: 'Seal',
                language: res.groups?.language ?? undefined,
            };
        }
        res = EmulatorCommon.doesLineMatch(line, EmulatorCommon.engageRegexes);
        if (res) {
            return {
                StartIn: '0',
                StartType: 'Engage',
                language: res.groups?.language ?? undefined,
            };
        }
    }
    static matchEnd(line) {
        let res;
        // Currently all of these regexes have groups if they match at all,
        // but be robust to that changing in the future.
        res = EmulatorCommon.doesLineMatch(line, EmulatorCommon.winRegex);
        if (res) {
            return {
                EndType: 'Win',
                language: res.groups?.language ?? undefined,
            };
        }
        res = EmulatorCommon.doesLineMatch(line, EmulatorCommon.wipeRegex);
        if (res) {
            return {
                EndType: 'Wipe',
                language: res.groups?.language ?? undefined,
            };
        }
        res = EmulatorCommon.doesLineMatch(line, EmulatorCommon.cactbotWipeRegex);
        if (res) {
            return {
                EndType: 'Cactbot Wipe',
                language: res.groups?.language ?? undefined,
            };
        }
        res = EmulatorCommon.doesLineMatch(line, EmulatorCommon.unsealRegexes);
        if (res) {
            return {
                EndType: 'Unseal',
                language: res.groups?.language ?? undefined,
            };
        }
    }
    static sealRegexes = LocaleNetRegex.areaSeal;
    static engageRegexes = LocaleNetRegex.countdownEngage;
    static countdownRegexes = LocaleNetRegex.countdownStart;
    static unsealRegexes = LocaleNetRegex.areaUnseal;
    static wipeRegex = commonNetRegex.wipe;
    static winRegex = NetRegexes.network6d({ command: '4000000[23]' });
    static cactbotWipeRegex = NetRegexes.echo({ line: 'cactbot wipe.*?' });
}
//# sourceMappingURL=EmulatorCommon.js.map