import Dexie from 'dexie';
import 'dexie-export-import';
import Encounter from './Encounter';
import LineEvent from './network_log_converter/LineEvent';
import PersistorEncounter from './PersistorEncounter';
export default class Persistor extends Dexie {
    encounters;
    encounterSummaries;
    constructor() {
        super('RaidEmulatorEncounters');
        this.version(1).stores({
            EncounterSummaries: '++id,start,zoneName,[zoneName+start]',
            Encounters: '++id',
        });
        this.encounters = this.table('Encounters');
        this.encounterSummaries = this.table('EncounterSummaries');
        this.encounterSummaries.mapToClass(PersistorEncounter);
        this.use({
            stack: 'dbcore',
            create: (down) => {
                return {
                    ...down,
                    table: (name) => {
                        const table = down.table(name);
                        if (name === 'Encounters') {
                            return {
                                ...table,
                                get: async (req) => {
                                    const obj = await table.get(req);
                                    Object.setPrototypeOf(obj, Encounter.prototype);
                                    for (const line of obj.logLines)
                                        Object.setPrototypeOf(line, LineEvent.prototype);
                                    // Check for encounter upgrade, re-save encounter if it's upgraded.
                                    if (obj.upgrade(obj.version)) {
                                        await this.persistEncounter(obj);
                                        return obj;
                                    }
                                    obj.initialize();
                                    return obj;
                                },
                            };
                        }
                        return table;
                    },
                };
            },
        });
    }
    async loadEncounter(id) {
        let enc;
        await this.transaction('readwrite', [this.encounters, this.encounterSummaries], async () => {
            enc = await this.encounters.get(id);
        });
        return enc;
    }
    async persistEncounter(baseEncounter) {
        const summary = new PersistorEncounter(baseEncounter);
        if (baseEncounter.id !== undefined) {
            await this.encounterSummaries.put(summary, baseEncounter.id);
            return this.encounters.put(baseEncounter, baseEncounter.id);
        }
        const id = await this.encounters.add(baseEncounter);
        baseEncounter.id = id;
        summary.id = id;
        return this.encounterSummaries.add(summary, id);
    }
    async deleteEncounter(id) {
        await this.encounterSummaries.delete(id);
        return this.encounters.delete(id);
    }
    async clearDB() {
        await this.encounters.clear();
        await this.encounterSummaries.clear();
    }
    async exportDB() {
        return this.export();
    }
    async importDB(file) {
        return this.import(file);
    }
}
//# sourceMappingURL=Persistor.js.map