import { UnreachableCode } from '../../../../resources/not_reached';
import StubbedPopupText from './StubbedPopupText';
export default class RaidEmulatorPopupText extends StubbedPopupText {
    $popupTextContainerWrapper;
    emulatedOffset;
    emulator;
    displayedText;
    scheduledTriggers;
    seeking;
    $textElementTemplate;
    audioDebugTextDuration;
    constructor(options, timelineLoader, raidbossFileData) {
        super(options, timelineLoader, raidbossFileData);
        const popupElem = document.querySelector('.popup-text-container-outer');
        if (!(popupElem instanceof HTMLElement))
            throw new UnreachableCode();
        this.$popupTextContainerWrapper = popupElem;
        this.emulatedOffset = 0;
        this.displayedText = [];
        this.scheduledTriggers = [];
        this.seeking = false;
        const templateElement = document.querySelector('template.textElement');
        if (!(templateElement instanceof HTMLTemplateElement))
            throw new UnreachableCode();
        const textElement = templateElement.content.firstElementChild;
        if (!(textElement instanceof HTMLElement))
            throw new UnreachableCode();
        this.$textElementTemplate = textElement;
        this.audioDebugTextDuration = 2000;
        const parentTtsSay = this.ttsSay;
        this.ttsSay = (ttsText) => {
            if (this.seeking)
                return;
            const div = this._makeTextElement(undefined, ttsText, 'tts-text');
            this.addDisplayText(div, this.emulatedOffset + this.audioDebugTextDuration);
            parentTtsSay(ttsText);
        };
    }
    async doUpdate(currentLogTime) {
        this.emulatedOffset = currentLogTime;
        for (const t of this.scheduledTriggers) {
            const remaining = t.expires - currentLogTime;
            if (remaining <= 0) {
                t.resolver();
                await t.promise;
            }
        }
        this.scheduledTriggers = this.scheduledTriggers.filter((t) => {
            return t.expires - currentLogTime > 0;
        });
        this.displayedText = this.displayedText.filter((t) => {
            const remaining = t.expires - currentLogTime;
            if (remaining > 0) {
                const elem = t.element.querySelector('.popup-text-remaining');
                if (!elem)
                    throw new UnreachableCode();
                elem.textContent = `(${(remaining / 1000).toFixed(1)})`;
                return true;
            }
            t.element.remove();
            return false;
        });
    }
    OnLog(_e) {
        throw new UnreachableCode();
    }
    onEmulatorLog(logs) {
        for (const l of logs) {
            const currentTime = l.timestamp;
            if (l.convertedLine.includes('00:0038:cactbot wipe'))
                this.SetInCombat(false);
            for (const trigger of this.triggers) {
                const r = trigger.localRegex?.exec(l.convertedLine);
                if (r)
                    this.OnTrigger(trigger, r, currentTime);
            }
            for (const trigger of this.netTriggers) {
                const r = trigger.localNetRegex?.exec(l.networkLine);
                if (r)
                    this.OnTrigger(trigger, r, currentTime);
            }
        }
    }
    OnNetLog(_e) {
        throw new UnreachableCode();
    }
    bindTo(emulator) {
        this.emulator = emulator;
        emulator.on('emitLogs', (event) => {
            this.onEmulatorLog(event.logs);
        });
        emulator.on('tick', async (currentLogTime) => {
            await this.doUpdate(currentLogTime);
        });
        emulator.on('midSeek', async (line) => {
            await this.doUpdate(line.timestamp);
        });
        emulator.on('preSeek', (seekTimestamp) => {
            this.seeking = true;
            this._emulatorReset();
            if (seekTimestamp < this.emulatedOffset)
                this.ReloadTimelines();
        });
        emulator.on('postSeek', () => {
            // This is a hacky fix for audio still playing during seek
            window.setTimeout(() => {
                this.seeking = false;
            }, 5);
        });
        emulator.on('currentEncounterChanged', () => {
            const currentEnc = emulator.currentEncounter;
            if (!currentEnc)
                return;
            const enc = currentEnc.encounter;
            if (!enc.encounterZoneName || enc.encounterZoneId === undefined)
                return;
            this._emulatorReset();
            this.OnChangeZone({
                type: 'ChangeZone',
                zoneName: enc.encounterZoneName,
                zoneID: parseInt(enc.encounterZoneId, 16),
            });
            // Always reset when changing encounters
            this.Reset();
        });
    }
    _emulatorReset() {
        for (const i of this.scheduledTriggers)
            i.rejecter();
        this.scheduledTriggers = [];
        this.displayedText = this.displayedText.filter((t) => {
            t.element.remove();
            return false;
        });
        this.triggerSuppress = {};
    }
    _createTextFor(triggerHelper, text, textType, _lowerTextKey, duration) {
        const textElementClass = `${textType}-text`;
        const e = this._makeTextElement(triggerHelper, text, textElementClass);
        this.addDisplayText(e, this.emulatedOffset + duration * 1000);
    }
    _onTriggerInternalDelaySeconds(triggerHelper) {
        const delay = 'delaySeconds' in triggerHelper.trigger
            ? triggerHelper.valueOrFunction(triggerHelper.trigger.delaySeconds)
            : 0;
        if (delay === undefined || delay === null || typeof delay !== 'number' || delay <= 0)
            return;
        let ret;
        // Disable prefer-const due to needing `ret` defined before using it in promise
        // eslint-disable-next-line prefer-const
        ret = new Promise((res, rej) => {
            this.scheduledTriggers.push({
                expires: this.emulatedOffset + delay * 1000,
                promise: ret,
                resolver: res,
                rejecter: rej,
            });
        });
        return ret;
    }
    _playAudioFile(triggerHelper, url, volume) {
        if (![this.options.InfoSound, this.options.AlertSound, this.options.AlarmSound]
            .includes(url)) {
            const div = this._makeTextElement(triggerHelper, url, 'audio-file');
            this.addDisplayText(div, this.emulatedOffset + this.audioDebugTextDuration);
        }
        if (this.seeking)
            return;
        super._playAudioFile(triggerHelper, url, volume);
    }
    _makeTextElement(_triggerHelper, text, className) {
        const $ret = this.$textElementTemplate.cloneNode(true);
        if (!($ret instanceof HTMLElement))
            throw new UnreachableCode();
        $ret.classList.add(className);
        const container = $ret.querySelector('.popup-text');
        if (!(container instanceof HTMLElement))
            throw new UnreachableCode();
        container.textContent = text;
        return $ret;
    }
    addDisplayText($e, endTimestamp) {
        const remaining = (endTimestamp - this.emulatedOffset) / 1000;
        const container = $e.querySelector('.popup-text-remaining');
        if (!(container instanceof HTMLElement))
            throw new UnreachableCode();
        container.textContent = `(${remaining.toFixed(1)})`;
        this.$popupTextContainerWrapper.append($e);
        this.displayedText.push({
            element: $e,
            expires: endTimestamp,
        });
    }
    setParserLanguage(lang) {
        this.parserLang = lang;
    }
}
//# sourceMappingURL=RaidEmulatorPopupText.js.map