import { parseString } from '@fast-csv/parse';
import fetch from 'node-fetch';
const baseGithub = 'https://raw.githubusercontent.com/';
const intlGithub = 'xivapi/ffxiv-datamining/master/csv/';
const cnGithub = 'thewakingsands/ffxiv-datamining-cn/master/';
const koGithub = 'Ra-Workspace/ffxiv-datamining-ko/master/csv/';
// Turn names from tables/xivapi into JavaScript-safe ascii string keys.
export const cleanName = (str) => {
    if (!str || str === '')
        return str;
    // The Tam\u2013Tara Deepcroft
    str = str.replace(/\u2013/g, '-');
    // The <Emphasis>Whorleater</Emphasis> Extreme
    str = str.replace(/<\/?Emphasis>/g, '');
    // Various symbols to get rid of.
    str = str.replace(/[':(),]/g, '');
    // Sigmascape V4.0 (Savage)
    str = str.replace(/\./g, '');
    // Common case hyphen: TheSecondCoilOfBahamutTurn1
    // Special case hyphen: ThePalaceOfTheDeadFloors1_10
    str = str.replace(/([0-9])-([0-9])/g, '$1_$2');
    str = str.replace(/[-]/g, ' ');
    // Of course capitalization isn't consistent, that'd be ridiculous.
    str = str.toLowerCase().replace(/(^[a-z])|([\s_]+[^a-z]*[a-z])/g, (match) => match.toUpperCase());
    // Now fix capitalization for the Containment Bay fights...
    str = str.replace(/(\s[A-Z][0-9][a-z][0-9])/g, (match) => match.toUpperCase());
    // ... and for V&C (Variant & Criterion)
    str = str.replace(/^V&c\s/g, 'V&C');
    // collapse remaining whitespace
    str = str.replace(/\s+/g, '');
    // remove non-ascii characters
    str = str.replace(/[^0-9A-z_]/g, '');
    return str;
};
export const readCsvContent = async (content) => {
    return await new Promise((resolve, reject) => {
        const rows = [];
        parseString(Array.isArray(content) ? content.join('\n') : content, {
            skipLines: 1,
            ignoreEmpty: false,
        })
            .on('error', (err) => reject(err))
            .on('data', (row) => rows.push(row))
            .on('end', (rowCount) => {
            if (rowCount === 0)
                reject(new Error('csv read no data'));
            const keys = rows.shift() ?? [];
            rows.shift();
            resolve({
                keys,
                rows,
            });
        });
    });
};
// inputs[0] is the key column for the returned map
export const makeMap = async (contents, inputs, _outputs) => {
    const { keys, rows } = await readCsvContent(contents);
    const indices = inputs.map((input) => {
        if (typeof input === 'number')
            return input;
        return keys.indexOf(input);
    });
    const outputs = _outputs ?? inputs;
    const map = Object.fromEntries(rows.map((row) => {
        const output = Object.fromEntries(indices.map((indice, index) => {
            return [outputs[index] ?? '', row[indice]];
        }));
        return [row[indices[0] ?? 0] ?? '', output];
    }));
    return map;
};
const getRemoteTable = async (url, inputs, outputs) => {
    const response = await fetch(url);
    const contents = await response.text();
    return await makeMap(contents, inputs, outputs);
};
export const getIntlTable = async (table, inputs, outputs) => {
    const url = `${baseGithub}${intlGithub}${table}.csv`;
    return getRemoteTable(url, inputs, outputs);
};
export const getKoTable = (table, inputs, outputs) => {
    const url = `${baseGithub}${koGithub}${table}.csv`;
    return getRemoteTable(url, inputs, outputs);
};
export const getCnTable = (table, inputs, outputs) => {
    const url = `${baseGithub}${cnGithub}${table}.csv`;
    return getRemoteTable(url, inputs, outputs);
};
export const getLocaleTable = async (table, locale, inputs, outputs) => {
    if (locale === 'cn')
        return getCnTable(table, inputs, outputs);
    else if (locale === 'ko')
        return getKoTable(table, inputs, outputs);
    throw new Error(`Invalid locale: ${locale}`);
};
export const getRawCsv = async (table, locale) => {
    let url = '';
    if (locale === 'cn')
        url = `${baseGithub}${cnGithub}${table}.csv`;
    else if (locale === 'ko')
        url = `${baseGithub}${koGithub}${table}.csv`;
    else
        throw new Error(`Invalid locale: ${locale}`);
    return await (await fetch(url)).text();
};
//# sourceMappingURL=csv_util.js.map