import { Namespace } from 'argparse';
import inquirer from 'inquirer';
import { UnreachableCode } from '../resources/not_reached';
import { ConsoleLogger, logLevels } from './console_logger';
import { default as generateEffectIds } from './gen_effect_id';
import { default as generateHunt } from './gen_hunt_data';
import { default as generatePetNames } from './gen_pet_names';
import { default as generateWeatherRate } from './gen_weather_rate';
import { default as generateWorldIds } from './gen_world_ids';
import { default as generateZoneIdandZoneInfo } from './gen_zone_id_and_info';
const fileKeys = [
    'effect_id',
    'hunt_data',
    'pet_names',
    'weather_rate',
    'world_id',
    'zone_id_and_info',
];
const allKey = 'all';
const allLabel = '* Generate All Data Files';
const fileKeyToFunc = {
    'effect_id': generateEffectIds,
    'hunt_data': generateHunt,
    'pet_names': generatePetNames,
    'weather_rate': generateWeatherRate,
    'world_id': generateWorldIds,
    'zone_id_and_info': generateZoneIdandZoneInfo,
};
const generateAll = async (logLevel) => {
    for (const key of fileKeys) {
        await fileKeyToFunc[key](logLevel);
    }
};
const fileChoices = [
    { name: allLabel, value: allKey },
    ...fileKeys.map((k) => ({ name: k, value: k })),
];
const fileKeyToFuncAll = ({ [allKey]: generateAll, ...fileKeyToFunc });
const logLevelChoices = logLevels.map((ll) => ({
    name: ll[1],
    value: ll[0],
}));
class GenerateDataFilesNamespace extends Namespace {
    'file';
    'loglevel';
}
const fileDefault = 'all';
const generateDataFilesFunc = async (args) => {
    if (!(args instanceof GenerateDataFilesNamespace))
        throw new UnreachableCode();
    const questions = [
        {
            type: 'list',
            name: 'file',
            message: 'Which data file do you want to generate?',
            choices: fileChoices,
            default: args.file,
            when: () => args.file === null || args.file === undefined,
        },
        {
            type: 'list',
            name: 'loglevel',
            message: 'What level of console logging do you want?',
            choices: logLevelChoices,
            default: args.loglevel,
            when: () => args.loglevel === null || args.loglevel === undefined,
        },
    ];
    return inquirer.prompt(questions)
        .then((answers) => {
        // args.file and args.loglevel return as objects, rather than string primitives. /shrug
        // when myLogLevel is passed to the gen_data function, it cannot recognize the log level,
        // so force it here to a string and then re-apply the type with an assertion.
        // parsearg already limits user-input values to the respective type literal.
        const myFile = answers.file ?? args.file ?? fileDefault;
        const myLogLevel = answers.loglevel ??
            args.loglevel ??
            ConsoleLogger.logLevelDefault;
        return fileKeyToFuncAll[myFile](myLogLevel.toString());
    }).catch(console.error);
};
export const registerGenerateDataFiles = (actionChoices, subparsers) => {
    actionChoices.generate = {
        name: 'Generate common data files',
        callback: generateDataFilesFunc,
        namespace: GenerateDataFilesNamespace,
    };
    const generateParser = subparsers.addParser('generate', {
        description: actionChoices.generate.name,
    });
    generateParser.addArgument(['-f', '--file'], {
        nargs: 1,
        type: 'string',
        choices: Object.keys(fileKeyToFuncAll),
        help: 'The data file to be generated (or \'all\')',
    });
    generateParser.addArgument(['-ll', '--loglevel'], {
        nargs: 1,
        type: 'string',
        choices: logLevels.map((ll) => ll[0]),
        help: 'The level of console output you want to see',
    });
};
//# sourceMappingURL=generate_data_files.js.map