import fetch from 'node-fetch';
const makeRequest = async (url, options) => {
    const requestUrl = `${url}?${options.toString()}`;
    const response = await fetch(requestUrl);
    const responseText = JSON.parse(await response.text());
    return responseText;
};
class FFLogs {
    static callFFLogs = async (fightsOrEvents, reportId, prefix, options) => {
        // The Event call requires specifying a View parameter.
        // There's a long list of these, but choosing "Summary"
        // permits us to filter more granularly via the option parameters.
        const urlSegment = fightsOrEvents === 'fights' ? fightsOrEvents : `${fightsOrEvents}/summary`;
        const url = `https://${prefix}.fflogs.com:443/v1/report/${urlSegment}/${reportId}`;
        let data = await makeRequest(url, options);
        // For a simple list of encounters, one call should be enough.
        if (fightsOrEvents === 'fights')
            return data;
        // For Event retrieval, check whether the data is paginated.
        // If it is, recursively retrieve it until it is all obtained.
        if (data.nextPageTimestamp !== undefined) {
            const nextOptions = new URLSearchParams(options);
            nextOptions.set('start', data.nextPageTimestamp.toString());
            data = data;
            const nextData = await this.callFFLogs('events', reportId, 'www', nextOptions);
            data.events = data.events.concat(nextData.events);
            data.count += nextData.count;
            data.nextPageTimestamp = nextData.nextPageTimestamp;
            return data;
        }
        return data;
    };
    static getFightInfo = async (reportId, key) => {
        const fightOptions = new URLSearchParams();
        fightOptions.set('api_key', key);
        const fightListData = await FFLogs.callFFLogs('fights', reportId, 'www', fightOptions);
        return fightListData;
    };
    static extractEnemiesFromReport = (fightData) => {
        const enemies = {};
        for (const enemy of fightData.enemies)
            enemies[enemy.id] = enemy.name;
        return enemies;
    };
    static getEventData = async (reportId, key, startTime, endTime, filter = 'source.disposition="enemy" and (type="cast" or type="begincast")', translate = true) => {
        const eventOptionParams = new URLSearchParams({
            'api_key': key,
            'start': startTime.toString(),
            'end': endTime.toString(),
            'filter': filter,
            'translate': translate.toString(),
        });
        const eventData = await this.callFFLogs('events', reportId, 'www', eventOptionParams);
        return eventData.events;
    };
    static parseFFLogsEvent = (event, enemies, startTime) => {
        const timestamp = startTime + event.timestamp;
        let combatant;
        if (event.source !== undefined)
            combatant = enemies[event.source.id];
        else if (event.sourceID !== undefined)
            combatant = enemies[event.sourceID];
        combatant ??= 'Unknown';
        const abilityId = event.ability.guid.toString(16).toUpperCase();
        let ability = event.ability.name;
        if (event.ability.name.toLowerCase().startsWith('unknown_'))
            ability = '--sync--';
        const entry = {
            timestamp: timestamp,
            combatant: combatant,
            abilityId: abilityId,
            abilityName: ability,
            type: event.type,
        };
        return entry;
    };
    static parseReport = async (reportId, fightIndex, apiKey) => {
        const rawReportData = await FFLogs.getFightInfo(reportId, apiKey);
        const reportStartTime = rawReportData.start;
        // First we verify that the user entered a valid index
        let chosenFight;
        for (const fight of rawReportData.fights) {
            if (fight.id === fightIndex)
                chosenFight = fight;
        }
        if (chosenFight === undefined)
            throw new Error(`No encounter found in report ${reportId} at index ${fightIndex}`);
        // Knowing that the encounter exists in the report,
        // we next assemble the list of all hostile entities in the report.
        // Unfortunately, entity data is not present in the data for individual encounters,
        // so we have to do some matching magic.
        const enemies = FFLogs.extractEnemiesFromReport(rawReportData);
        const rawFightData = await FFLogs.getEventData(reportId, apiKey, chosenFight.start_time, chosenFight.end_time);
        const startTime = chosenFight.start_time + reportStartTime;
        return rawFightData.map((event) => this.parseFFLogsEvent(event, enemies, startTime));
    };
}
export default FFLogs;
//# sourceMappingURL=fflogs.js.map